//----------------------------------------------------------------------------
//
//  This file is part of seq24.
//
//  seq24 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  seq24 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with seq24; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//-----------------------------------------------------------------------------
#include "event.h"
#include "perftime.h"


perftime::perftime( perform *a_perf, Adjustment *a_hadjust ): DrawingArea() 
{     
    m_mainperf = a_perf;

    add_events( GDK_BUTTON_PRESS_MASK |  
		GDK_BUTTON_RELEASE_MASK );

    // in the construor you can only allocate colors, 
    // get_window() returns 0 because we have not be realized
    Gdk_Colormap colormap = get_default_colormap();

    m_black = Gdk_Color( "black" );
    m_white = Gdk_Color( "white" );
    m_grey  = Gdk_Color( "grey" );

    m_hadjust = a_hadjust;

    colormap.alloc( m_black );
    colormap.alloc( m_white );
    colormap.alloc( m_grey );

    m_4bar_offset = 0;

    m_hadjust->value_changed.connect( slot( this, &perftime::change_horz ));
} 


void
perftime::increment_size()
{

}

void 
perftime::update_sizes()
{
 
}

void 
perftime::realize_impl()
{
    // we need to do the default realize
    Gtk::DrawingArea::realize_impl();

    // Now we can allocate any additional resources we need
    m_window = get_window();
    m_gc.create( m_window );
    m_window.clear();

    size( 10, c_timearea_y );
}


void
perftime::change_horz( )
{
    if ( m_4bar_offset != (int) m_hadjust->get_value() ){
	
	m_4bar_offset = (int) m_hadjust->get_value();
	queue_draw();
    }
}



int 
perftime::idle_progress( )
{
    return true;
}



void 
perftime::update_pixmap()
{

}




void 
perftime::draw_pixmap_on_window()
{

}

int 
perftime::expose_event_impl(GdkEventExpose* a_e)
{
    /* clear background */
    m_gc.set_foreground(m_white);
    m_window.draw_rectangle(m_gc,true,
			    0,
			    0, 
			    m_window_x, 
			    m_window_y );

    m_gc.set_foreground(m_black);
    m_window.draw_line(m_gc,
		       0,
		       m_window_y - 1,
		       m_window_x,
		       m_window_y - 1 );
    
    
    /* draw vert lines */
    m_gc.set_foreground(m_grey);
    for ( int i=0; i< (m_window_x * c_perf_scale_x / (c_ppqn * 4)) + 1; i++ )
    {
	/* beat */
	m_window.draw_line(m_gc,
			   i * c_ppqn * 4 / c_perf_scale_x,
			   0,
			   i * c_ppqn * 4 / c_perf_scale_x,
			   m_window_y );
	
	char bar[5];
	sprintf( bar, "%d", i + m_4bar_offset * 4 ); 
	
	m_gc.set_foreground(m_black);
	m_window.draw_string(c_text_font_6_12, m_gc, 
			     (i * c_ppqn * 4 / c_perf_scale_x) + 2, 
			     8,
			     bar );
    }

//     long start = m_mainperf->get_left_tick( );
    
//     /* adjust */
//     start -= (m_4bar_offset * 16 * c_ppqn);
//     start /= c_perf_scale_x;


//     if ( start >= 0 && start <= m_window_x ){

// 	m_gc.set_foreground(m_black);
// 	m_window.draw_rectangle(m_gc,true,
// 				    start,
// 				    0, 
// 				    7, 
// 				    10 );

// 	m_gc.set_foreground(m_white);
// 	m_window.draw_string(c_text_font_6_12, m_gc, 
// 			     start + 1, 
// 			     8,
// 			     "S" );
//     }

    long left = m_mainperf->get_left_tick( );
    long right = m_mainperf->get_right_tick( );

    left -= (m_4bar_offset * 16 * c_ppqn);
    left /= c_perf_scale_x;
    right -= (m_4bar_offset * 16 * c_ppqn);
    right /= c_perf_scale_x;

    if ( left >=0 && left <= m_window_x ){

	m_gc.set_foreground(m_black);
	m_window.draw_rectangle(m_gc,true,
				    left, m_window_y - 9, 
				    7, 
				    10 );

	m_gc.set_foreground(m_white);
	m_window.draw_string(c_text_font_6_12, m_gc, 
			     left + 1, m_window_y - 1,
			     "L" );

    }

    if ( right >=0 && right <= m_window_x ){
	
	m_gc.set_foreground(m_black);
	m_window.draw_rectangle(m_gc,true,
				    right - 6, m_window_y - 9, 
				    7, 
				    10 );

	m_gc.set_foreground(m_white);
	m_window.draw_string(c_text_font_6_12, m_gc, 
			     right - 6 + 1, m_window_y - 1,
			     "R" );

    }

    return true;
}

int 
perftime::button_press_event_impl(GdkEventButton* p0)
{
    long tick = (long) p0->x;
    tick *= c_perf_scale_x;
    tick = tick - (tick % (c_ppqn * 4));
    tick += (m_4bar_offset * 16 * c_ppqn);

    //if ( p0->button == 2 )
    //m_mainperf->set_start_tick( tick );
    if ( p0->button == 1 )
	m_mainperf->set_left_tick( tick );
    if ( p0->button == 3 )
	m_mainperf->set_right_tick( tick + (c_ppqn * 4) );

    queue_draw();


    return true;
}

int 
perftime::button_release_event_impl(GdkEventButton* p0)
{
    return false;
}



void
perftime::size_allocate_impl(GtkAllocation* a_r )
{
    Gtk::DrawingArea::size_allocate_impl( a_r );

    m_window_x = a_r->width;
    m_window_y = a_r->height;
}
